@@ -47,7 +47,8 @@
 #' As a rule of thumb, there is no evidence to suggest a forecasting model is
 #' miscalibrated if the p-value found was greater than a threshold of p >= 0.1,
 #' some evidence that it was miscalibrated if 0.01 < p < 0.1, and good
-#' evidence that it was miscalibrated if p <= 0.01.
+#' evidence that it was miscalibrated if p <= 0.01. However, the AD-p-values
+#' may be overly strict and there actual usefulness may be questionable.
 #' In this context it should be noted, though, that uniformity of the
 #' PIT is a necessary but not sufficient condition of calibration.
 #'
@@ -75,9 +76,8 @@
 #' \item \code{hist_PIT} a plot object with the PIT histogram. Only returned
 #' if \code{plot == TRUE}. Call
 #' \code{plot(PIT(...)$hist_PIT)} to display the histogram.
-#' \item \code{p_values}: all p_values generated from the Anderson-Darling tests on the
-#' randomised PIT. Only returned for integer forecasts
-#' and if \code{full_output = TRUE}
+#' \item \code{p_values}: all p_values generated from the Anderson-Darling tests
+#' on the (randomised) PIT. Only returned if \code{full_output = TRUE}
 #' \item \code{u}: the u_t values internally computed. Only returned if
 #' \code{full_output = TRUE}
 #' }
@@ -107,35 +107,27 @@ pit <- function(true_values,
                 predictions,
                 plot = TRUE,
                 full_output = FALSE,
-                n_replicates = 20,
+                n_replicates = 50,
                 num_bins = NULL,
                 verbose = FALSE) {
 
 
 
-  # ============== Error handling ==============
-
-  if (missing(true_values) | missing(predictions)) {
-    stop("true_values or predictions argument missing")
-  }
-
-  ## check whether continuous or integer
-  if (all.equal(as.vector(predictions), as.integer(predictions)) != TRUE) {
-    continuous_predictions <- TRUE
-  } else {
-    continuous_predictions <- FALSE
+  # error handling--------------------------------------------------------------
+  # check al arguments are provided
+  if (!all(c(methods::hasArg("true_values"), methods::hasArg("predictions")))) {
+    stop("`true_values` or `predictions` missing in function 'pit()'")
   }
+  check_not_null(true_values = true_values, predictions = predictions)
 
+  # check if there is more than one observation
   n <- length(true_values)
-
-
   if (n == 1) {
     if (verbose) {
       message("you need more than one observation to assess uniformity of the PIT")
     }
     out <- list(p_value = NA,
                 sd = NA)
-
     if (full_output) {
       out <- list(p_values = NA,
                   calibration = NA,
@@ -143,6 +135,7 @@ pit <- function(true_values,
     }
   }
 
+  # check and handle format of predictions
   if (is.data.frame(predictions)) {
     predictions <- as.matrix(predictions)
   }
@@ -158,8 +151,17 @@ pit <- function(true_values,
     stop(msg)
   }
 
-  # ============================================
 
+  # check data type ------------------------------------------------------------
+  # check whether continuous or integer
+  if (all.equal(as.vector(predictions), as.integer(predictions)) != TRUE) {
+    continuous_predictions <- TRUE
+  } else {
+    continuous_predictions <- FALSE
+  }
+
+
+  # calculate PIT --------------------------------------------------------------
   n_pred <- ncol(predictions)
 
   # calculate emipirical cumulative distribution function as
@@ -170,32 +172,32 @@ pit <- function(true_values,
                 },
                 .0)
 
+  # calculate PIT for continuous predictions case
   if (continuous_predictions) {
     p_value <- goftest::ad.test(P_x)$p.value
-
     out <- list(p_value = p_value,
                 sd = NA)
-
     if (plot) {
       hist_PIT <- hist_PIT(P_x, num_bins = num_bins, caption = p_value)
       out$hist_PIT = hist_PIT
     }
-
     if(full_output) {
       out$u <- P_x
+      out$p_values <- p_value
     }
+  }
 
-  } else {
+  # calculate PIT for integer predictions case
+  if (!continuous_predictions) {
     # empirical cdf for (y-1) for integer-valued predictions
     P_xm1 <- vapply(seq_along(true_values),
                     function(i) {
                       sum(predictions[i,] <= true_values[i] - 1) / n_pred
                     },
                     .0)
-
     # do n_replicates times for randomised PIT
     u <- replicate(n_replicates, P_xm1 + stats::runif(n) * (P_x - P_xm1))
-
+    # apply Anderson Darling test on u values
     p_values <- apply(
       u,
       MARGIN = 2,
@@ -203,19 +205,14 @@ pit <- function(true_values,
         goftest::ad.test(x)$p.value
       }
     )
-
     out <- list(p_value = mean(p_values),
                 sd = stats::sd(p_values))
-
+    # add additional output if desired
     if (full_output) {
       out$u <- u
       out$p_values <- p_values
-      out <- list(p_values = p_values,
-                  calibration = calibration,
-                  u = u)
     }
-
-
+    # make plot if desired
     if (plot) {
       hist_PIT <- hist_PIT(rowMeans(u), num_bins = num_bins,
                            caption = mean(p_values))
@@ -231,6 +228,166 @@ pit <- function(true_values,
 
 
 
+
+#' @title Probability Integral Transformation (data.frame Format)
+#'
+#' @description Wrapper around `pit()` for use in data.frames
+#'
+#' @details
+#' see \code{\link{pit}}
+#'
+#' @param data a data.frame with the following columns: `true_value`,
+#' `prediction`, `sample`
+#' @inheritParams pit
+#' @return a list with the following components:
+#' \itemize{
+#' \item \code{data}: the input data.frame (not including rows where prediction is `NA`),
+#' with added columns `pit_p_val` and `pit_sd`
+#' \item \code{hist_PIT} a plot object with the PIT histogram. Only returned
+#' if \code{plot == TRUE}. Call
+#' \code{plot(PIT(...)$hist_PIT)} to display the histogram.
+#' \item \code{p_values}: all p_values generated from the Anderson-Darling tests on the
+#' (randomised) PIT. Only returned if \code{full_output = TRUE}
+#' \item \code{u}: the u_t values internally computed. Only returned if
+#' \code{full_output = TRUE}
+#' }
+#' @importFrom goftest ad.test
+#' @importFrom stats runif sd
+#' @examples
+#' example <- scoringutils::continuous_example_data
+#' result <- pit_df(example, full_output = TRUE)
+#'
+#' @export
+#' @references
+#' Sebastian Funk, Anton Camacho, Adam J. Kucharski, Rachel Lowe,
+#' Rosalind M. Eggo, W. John Edmunds (2019) Assessing the performance of
+#' real-time epidemic forecasts: A case study of Ebola in the Western Area
+#' region of Sierra Leone, 2014-15, <doi:10.1371/journal.pcbi.1006785>
+
+pit_df <- function(data,
+                   plot = TRUE,
+                   full_output = FALSE,
+                   n_replicates = 100,
+                   num_bins = NULL,
+                   verbose = FALSE) {
+
+  data <- data.table::as.data.table(data)
+
+  # filter out instances where prediction is NA
+  data <- data[!is.na(prediction)]
+
+  # reformat data.table to wide format for PIT
+  data_wide <- data.table::dcast(data, ... ~ paste("sampl_", sample, sep = ""),
+                                 value.var = "prediction")
+
+  samples <- as.matrix(data_wide[, grepl("sampl_", colnames(data_wide)),
+                                 with = FALSE])
+  # extract true values
+  true_values <- data_wide$true_value
+
+  pit_arguments = list(true_values = true_values,
+                       predictions = samples,
+                       plot = plot,
+                       full_output = full_output,
+                       n_replicates = n_replicates,
+                       num_bins = num_bins,
+                       verbose = verbose)
+
+  # call pit with samples and true values
+  res <- do.call(pit, pit_arguments)
+
+  # add results back to the data.frame
+  data[, `:=` (pit_p_val = res$p_value,
+               pit_sd = res$sd)]
+
+  out <- list(data = data,
+              hist_PIT = res$hist_PIT)
+
+  if (full_output) {
+    out$p_values <- res$p_values
+    out$u <- res$u
+  }
+
+  return(out)
+}
+
+
+
+
+
+#' @title Probability Integral Transformation (data.frame Format, fast version)
+#'
+#' @description Wrapper around `pit()` for fast use in data.frames. This version
+#' of the pit does not do allow any plotting, but can iterate over categories
+#' in a data.frame as specified in the `by` argument.
+#'
+#' @details
+#' see \code{\link{pit}}
+#'
+#' @param data a data.frame with the following columns: `true_value`,
+#' `prediction`, `sample`
+#' @inheritParams pit
+#' @param by character vector with categories to iterate over
+#' @return the input data.frame (not including rows where prediction is `NA`),
+#' with added columns `pit_p_val` and `pit_sd`
+#' @importFrom goftest ad.test
+#' @importFrom stats runif sd
+#' @examples
+#' example <- scoringutils::continuous_example_data
+#' result <- pit_df(example, full_output = TRUE)
+#'
+#' @export
+#' @references
+#' Sebastian Funk, Anton Camacho, Adam J. Kucharski, Rachel Lowe,
+#' Rosalind M. Eggo, W. John Edmunds (2019) Assessing the performance of
+#' real-time epidemic forecasts: A case study of Ebola in the Western Area
+#' region of Sierra Leone, 2014-15, <doi:10.1371/journal.pcbi.1006785>
+
+pit_df_fast <- function(data,
+                        n_replicates = 100,
+                        by = by) {
+
+  data <- data.table::as.data.table(data)
+
+  # filter out instances where prediction is NA
+  data <- data[!is.na(prediction)]
+
+  # define arguments for call to PIT function
+  pit_arguments = list(plot = FALSE,
+                       full_output = FALSE,
+                       n_replicates = n_replicates,
+                       num_bins = 1,
+                       verbose = FALSE)
+
+  # reformat data.table to wide format for PIT
+  data_wide <- data.table::dcast(data, ... ~ paste("sampl_", sample, sep = ""),
+                                 value.var = "prediction")
+
+  # calculate PIT values
+  data_wide[, c("pit_p_val", "pit_sd") := do.call(pit, c(list(true_value,
+                                                        as.matrix(.SD)),
+                                                   pit_arguments)),
+      .SDcols = names(data_wide)[grepl("sampl_", names(data_wide))], by = by]
+
+  # melt data back
+  sample_names <- names(data_wide)[grepl("sampl_", names(data_wide))]
+  data <- data.table::melt(data_wide,
+                   measure.vars = sample_names,
+                   variable.name = "sample",
+                   value.name = "prediction")
+
+
+  return(data)
+}
+
+
+
+
+
+
+
+
+
 #' @title PIT Histogram
 #'
 #' @description
@@ -255,7 +412,7 @@ hist_PIT <- function(PIT_samples,
   }
 
   hist_PIT <- ggplot2::ggplot(data = data.frame(x = PIT_samples),
-                  ggplot2::aes(x = x)) +
+                              ggplot2::aes(x = x)) +
     ggplot2::geom_histogram(ggplot2::aes(y = stat(count) / sum(count)),
                             breaks = seq(0, 1, length.out = num_bins + 1),
                             colour = "grey") +
@@ -284,8 +441,8 @@ hist_PIT <- function(PIT_samples,
 
 
 hist_PIT_quantile <- function(PIT_samples,
-                     num_bins = NULL,
-                     caption = NULL) {
+                              num_bins = NULL,
+                              caption = NULL) {
 
   if (is.null(num_bins)) {
     n <- length(PIT_samples)
